#include "gtfcreate.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"

GTFcreate::GTFcreate(QWidget *parent) :
    QMainWindow(parent)
{
}



//---------------------------------------------------------//
// 分離伝達関数作成
//---------------------------------------------------------//
void MainWindow::GTFcreate()
{


    try
    {

        GTFDispClear();//結果表示画面クリア


        // 数字パラメータ格納
        m_param.NFFT                 = ui->gtf_fft_length->value();    //FFT長
        m_param.NFS                  = ui->gtf_sampling_freq->value(); //サンプリング周波数
        m_param.IMPULSE_RESPONSE_LEN = ui->gtf_tsp_length->value();    //tsp長
        m_param.geometry_calculus    = ui->gtf_geometry->isChecked();  //幾何計算

        std::cout << "gtf_geometry->isChecked()=" << m_param.geometry_calculus << std::endl;


        //-------------------------------------------------------------------//
        // harkio  領域確保
        //-------------------------------------------------------------------//
        if(m_plotctl[m_out_page_no].harkio != NULL)
        {
            qDebug() << __func__ << "delete harkio" ;

            delete m_plotctl[m_out_page_no].harkio;
            m_plotctl[m_out_page_no].harkio = NULL;
        }

        m_plotctl[m_out_page_no].harkio = new harkio::HARK;



        //-------------------------------------------------------------------//
        // MICARY-LocationFile のチェック
        //-------------------------------------------------------------------//
        qDebug() << "MICARY-LocationFile のチェック";

        std::string fname;
        fname = ui->gtf_mic_file->text().toStdString();

        qDebug() << "fname=" << fname.c_str() ;


        if (fname == "")
        {
               THROW_CRITICAL(tr("Input MICARY-LcationFile", "MICARY-LcationFileを入力してください"));
                return;
        }

        auto mic_file_type = GetHarkConfigFileType(fname);

        if (mic_file_type.second == HarkConfigFileType::Unknown)
        {
           THROW_CRITICAL(tr("Unknown format:MicArrayLcationFile", "MicArrayLcationFileが不明なフォーマットです"));
           return;
        }
        else
        {
            if (mic_file_type.second != HarkConfigFileType::MicArrayLocationFile)
            {
                THROW_CRITICAL(tr("Invalid file type:MicArrayLcationFile", "MicArrayLcationFileが不正なファイルタイプです"));
                return;
            }
        }


        if (mic_file_type.first != HarkConfigFileFormatType::XML)
        {
            THROW_CRITICAL("Not XML file:MicArrayLcationFile");
            return;
        }



        m_param.mic_file = fname; // MicArrayLcationFileを格納


       qDebug() << "param.geometry_calculus=" << m_param.geometry_calculus ;


        //-------------------------------------------------------------------//
        // ImpulseResponse-ListFIle, TSP-ListFile のチェック
        //-------------------------------------------------------------------//
        fname = ui->gtf_list_file->text().toStdString();

        qDebug() << "fname=" << fname.c_str() ;


        if (fname == "")
        {
            THROW_CRITICAL(tr("Input TSP/IRListFile", "TSP/IRListFileを入力してください"));
            return;
        }


        //ファイルタイプ確認
        auto src_file_type = GetHarkConfigFileType(fname);


        if (src_file_type.second == HarkConfigFileType::Unknown)
        {
            THROW_CRITICAL(tr("Unknown format:%1", "%1が不明なフォーマットです").arg("TSP/IRListFile"));
            return;
        }
        else if (src_file_type.first != mic_file_type.first)
        {
            THROW_CRITICAL(tr("MicArrayLocationFile and %1 have to be the same format (Text or XML)",
                                "MicArrayLocationFileと%1は同じフォーマットでなければいけません (Text または XML)" )
                    .arg("TSP/IRListFile"));
            return;
        }
        else if (src_file_type.second != HarkConfigFileType::ImpulseResponseListFile &&
                 src_file_type.second != HarkConfigFileType::TSPListFile)
        {
            THROW_CRITICAL(tr("Invalid file type:%1", "%1が不正なファイルタイプです").arg("TSP/IRListFile"));
            return;
        }
        else if ((ui->check_impulse->isChecked() &&
                    src_file_type.second != HarkConfigFileType::ImpulseResponseListFile) ||
                ((ui->check_tsp->isChecked() &&
                    src_file_type.second != HarkConfigFileType::TSPListFile)))
        {
            MainWindow::THROW_CRITICAL(tr("Invalid file type:%1", "%1が不正なファイルタイプです").arg("TSP/IRListFile"));
            return;
        }


        //file type XMLチェック
        if (src_file_type.first != HarkConfigFileFormatType::XML)
        {
            MainWindow::THROW_CRITICAL("Not XML file:TSP/IRListFile");
            return;
        }

        m_param.src_file = fname ; // IR/TSP listファイル設定


        // 一時ディレクトリを生成
        boost::filesystem::path temp_dir = CreateTemporaryDirectory();
        boost::filesystem::path hark_file_path = temp_dir / "hark.tf";



        //-----------------------------------------------------------------//
        //  正規化設定
        //-----------------------------------------------------------------//
        QString strNorm;

        strNorm="";

        if(ui->checkBox_1->isChecked() == TRUE)
        {
            m_param.norm_non =1;
            strNorm = "" ;
        }

        if(ui->checkBox_2->isChecked() == TRUE)
        {
            m_param.norm_src = 1;
            strNorm = "--normalize-src " ;
        }

        if(ui->checkBox_3->isChecked() == TRUE)
        {
            m_param.norm_mic = 1;
            strNorm = strNorm + "--normalize-mic ";
        }

        if(ui->checkBox_4->isChecked() == TRUE)
        {
            m_param.norm_freq = 1;
            strNorm = strNorm + "--normalize-freq ";
        }




        printf("*---------------------------------------------------------*\n");

        printf("harktool4_cui \n"
                    "--mode tf \n"
                    "--output-file %s \n"
                    "--output-format binary \n"
                    "--output-type gtf \n"
                    "--nfft %d \n"
                    "%s \n"
                    "--sampling-freq %d \n"
                    "%s \n"   // geometry_calculus
                    " %s \n"   // mic_file
                    " %s \n"   // src_file
                    , "hark.tf"
                    ,m_param.NFFT
                    ,strNorm.toStdString().c_str()
                    ,m_param.NFS
                    ,(m_param.geometry_calculus ? "--geometry-calculus 1" : "")
                    , m_param.mic_file.c_str()
                    , m_param.src_file.c_str() );

            printf("*---------------------------------------------------------*\n");



        QString command = (boost::format("harktool4_cui "
                            "--mode tf "
                            "--output-file %s "
                            "--output-format binary "
                            "--output-type gtf "
                            "--nfft %d "
                            "%s "
                            "--sampling-freq %d "
                            "%s "
                            "%s "
                            "%s " )
                            % hark_file_path
                            % m_param.NFFT
                            % strNorm.toStdString().c_str()
                            % m_param.NFS
                            % (m_param.geometry_calculus ? "--geometry-calculus 1" : "")
                            % m_param.mic_file
                            % m_param.src_file).str().c_str();


        qDebug() << "command = " << command ;

        //run status設定
        setRunStatus(RUN_GTF_CUI);


        //       command="xxxharktool";

        m_Process.start(command);//harktool4_cui実行

        qDebug() << "m_Process.pid=" << m_Process.pid();


        std::cout << "hark_file_path=" << hark_file_path.native().c_str() << std::endl;

        m_plotctl[m_out_page_no].hark_file_path = hark_file_path;

        MainWindow::m_bar = 0;
        ui->progressBar->setValue(0);
        ui->progressBar->setRange(0,72);
        ui->stackedWidget->setCurrentIndex(DISP_scrProgressBar);//表示

        setPageNo(RUN_GTF_CUI);//表示画面番号格納

        MainWindow::setCheckLabel();//chek中label表示


        QTimer::singleShot( 5000, this, SLOT(ErrorMsg()) );

        qDebug("CalcSys end");


    }
    catch (const std::exception& e)
    {
         THROW_CRITICAL(e.what());
    }
}







//---------------------------------------------------------//
//
//---------------------------------------------------------//
void MainWindow::GTFDataDisp()
{

    // 作成したGTF伝達関数を読み込む
    qDebug() << "GTFDataDisp(作成した分離伝達関数を読み込む)";


    std::cout << "hark_file_path=" << m_plotctl[m_out_page_no].hark_file_path.native().c_str() << std::endl;

    boost::uintmax_t size = boost::filesystem::file_size(m_plotctl[m_out_page_no].hark_file_path.native().c_str());

    std::cout << "File size= " << size << std::endl;

    if(size <=0 )
    {
        MainWindow::THROW_CRITICAL("Can not Create harkfile");
        return;
    }

     m_plotctl[m_out_page_no].harkio->readFile(m_plotctl[m_out_page_no].hark_file_path.native()); //harkio::HARKに格納


    // SPECデータの上書き
    harkio::SPEC* spec = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::SPEC> ();
    spec->setDate(QDateTime::currentDateTime().toString("yyyy/MM/dd").toStdString().c_str());
    spec->setCreator(getUserName().toStdString().c_str());





    //-----------------------------------------------------------------//
    // 結果表示
    //-----------------------------------------------------------------//
    //編集不可に設定
    ui->gtf_table->setEditTriggers(QAbstractItemView::NoEditTriggers);


    //        ui->gtf_table->setItem(行,列,itm);

    //------------------------------------------------------//
    // fmt chunk
    //------------------------------------------------------//
    QTableWidgetItem* item;
    QString value;
    int row = 0; //fmt開始行

    harkio::FMT *fmt = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::FMT>();

    //fmtバージョン
    value = QString("%1").arg(fmt->getVersionStr().c_str());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み



    //Format ID
    value = QString("%1").arg(fmt->getFormatId());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    //mic数
    value = QString("%1").arg(fmt->getNMic());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);




    //FFT長
    value = QString("%1").arg(fmt->getNFFT());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    //インパルスレスポンス長
    value = QString("%1").arg(fmt->getNIR());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    //サンプリング周波数
    value = QString("%1").arg(fmt->getNFS());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    //Mic Position
    item = new QTableWidgetItem("Plot");
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    ++row; //空白行分
    ++row; //GTF表題分

    //------------------------------------------------------//
    // GTF chunk
    //------------------------------------------------------//
    harkio::GTF *gtf = m_plotctl[m_out_page_no].harkio->getData()->getChunk<harkio::GTF>();



    //gtfバージョン
    value = QString("%1").arg(gtf->getVersionStr().c_str());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み


    //Num of Source
    value = QString("%1").arg(gtf->getNSrc());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row ,1,item);//1行,1列書き込み



    //Source Location
    item = new QTableWidgetItem("Plot");
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    //Trancfer Function
    item = new QTableWidgetItem("Plot");
    item->setForeground(Qt::white);
    item->setFont(QFont("Ubuntu",12,QFont::Bold));
    ui->gtf_table->setItem(++row,1,item);


    ++row; //空白行分
    ++row; //SPEC表題分

    //------------------------------------------------------//
    // SPEC chunk
    //------------------------------------------------------//

    //作成日
    value = QString("%1").arg(spec->getDate());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み


    //場所
    value = QString("%1").arg(spec->getLocation());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み


    //ロボット
    value = QString("%1").arg(spec->getRobot());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み

    //アレイのタイプ
    value = QString("%1").arg(spec->getArrayType());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み


    //作成者
    value = QString("%1").arg(spec->getCreator());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み


    //備考
    value = QString("%1").arg(spec->getMemo());
    item = new QTableWidgetItem(value);
    item->setForeground(Qt::white); //文字色
    item->setFont(QFont("Ubuntu",12,QFont::Bold));//文字style
    ui->gtf_table->setItem(++row,1,item);//1行,1列書き込み



    //------------------------------------------------------//
    // harkioの値をGNUPlot.hに設定する
    //------------------------------------------------------//
    m_plotctl[m_out_page_no].param.NMic =  fmt->getNMic();  //mic数
    m_plotctl[m_out_page_no].param.NFFT =  fmt->getNFFT(); // FFT長
    m_plotctl[m_out_page_no].param.NIR  =  fmt->getNIR();  // インパルスレスポンス長
    m_plotctl[m_out_page_no].param.NFS  =  fmt->getNFS();  //サンプリング周波数

    m_plotctl[m_out_page_no].param.GTF_NSRC =  gtf->getNSrc();
    m_plotctl[m_out_page_no].param.GTF_NFI =  gtf->getNFI(); //周波数ライン数
}




//---------------------------------------------------------//
// hrkio表示クリア
//---------------------------------------------------------//
void MainWindow::GTFDispClear()
{

    int i;

    //FMTチャンク初期化
    for( i = DISP_FMT_VERSION ; i <= DISP_MIC_POSITION ; i++)
    {
         ui->gtf_table->setItem( i,1, new QTableWidgetItem(""));//空白
    }

    //M２PGチャンク初期化
    for( i = DISP_GTF_VERSION ; i <= DISP_TRANSFER_FUNCTION ; i++)
    {
         ui->gtf_table->setItem( i ,1, new QTableWidgetItem(""));//空白
    }

    //SPECチャンク初期化
    for( i = DISP_DATA ; i <= DISP_NOTES ; i++)
    {
         ui->gtf_table->setItem( i ,1, new QTableWidgetItem(""));//空白
    }

}



